module QC where

import Test.QuickCheck
import qualified Data.Integer as Pure

arbitraryInteger :: Gen String
arbitraryInteger =
    sized $ \n-> fmap (concatMap show)
                 $ sequence [ choose (0,9::Int) | _ <- [0..n*2] ]


prop_readShow :: Property
prop_readShow =
    forAll arbitraryInteger $ \i ->
        show (read i::Integer)
        ==
        show (read i::Pure.Integer)

prop_mult :: Property
prop_mult =
    forAll arbitraryInteger $ \i ->
        forAll arbitraryInteger $ \j ->
            (show (read i * read j ::Integer))
            ==
            (show (read i * read j ::Pure.Integer))

prop_add :: Property
prop_add =
    forAll arbitraryInteger $ \i ->
        forAll arbitraryInteger $ \j ->
            (show (read i + read j ::Integer))
            ==
            (show (read i + read j ::Pure.Integer))

prop_sub :: Property
prop_sub =
    forAll arbitraryInteger $ \i ->
        forAll arbitraryInteger $ \j ->
            (show (read i - read j ::Integer))
            ==
            (show (read i - read j ::Pure.Integer))

prop_quot :: Property
prop_quot =
    forAll arbitraryInteger $ \i ->
        forAll arbitraryInteger $ \j ->
            let (k,l) = if i>j then (i,j) else (j,i) in
            ((read l :: Integer) /=0) ==>
              (show (read k `quotRem` (read l ::Integer)))
              ==
              (show (read k `quotRem` (read l ::Pure.Integer)))

prop_quot2 :: Property
prop_quot2 =
    forAll arbitraryInteger $ \i ->
        forAll arbitraryInteger $ \j ->
            let prodI = (read i * (read j ::Integer))
                prodP = (read i * (read j ::Pure.Integer))
            in
            (show (prodI `quotRem` (read i ::Integer)))
            ==
            (show (prodP `quotRem` (read i ::Pure.Integer)))

main = do
    putStr "prop_readShow "; quickCheck prop_readShow
    putStr "prop_add      "; quickCheck prop_add
    putStr "prop_sub      "; quickCheck prop_sub
    putStr "prop_mult     "; quickCheck prop_mult
    putStr "prop_quot     "; quickCheck prop_quot
    putStr "prop_quot2    "; quickCheck prop_quot2
